package com.example.greenwatch;

import androidx.appcompat.app.AppCompatActivity;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.example.greenwatch.adapters.AlarmHistoryListAdapter;
import com.example.greenwatch.adapters.DeviceListAdapter;
import com.example.greenwatch.models.Device;
import com.example.greenwatch.viewmodels.AccelerometerViewModel;

import java.util.List;

public class AccelerometerActivity extends AppCompatActivity implements SensorEventListener {

    private SensorManager accelerometerManager;
    private Sensor accelerometerSensor;
    private TextView accelerometerStatusMessage;
    private TextView accelerometerDataTV;
    private TextView accelerometerWarningTV;
    private Button backToMainActivityButton;
    private AccelerometerViewModel mAccelerometerViewModel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_accelerometer);

        accelerometerStatusMessage = (TextView) findViewById(R.id.tvAccelerometerStatusmessage);
        accelerometerDataTV = (TextView) findViewById(R.id.tvAccelerometerData);
        accelerometerWarningTV = (TextView) findViewById(R.id.tvAccelerometerWarning);
        backToMainActivityButton = (Button) findViewById(R.id.accelerometerBackToMainActivity);

        RecyclerView recyclerView = findViewById(R.id.deviceListRecyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setHasFixedSize(true);

        final DeviceListAdapter deviceListAdapter = new DeviceListAdapter();
        recyclerView.setAdapter(deviceListAdapter);

        RecyclerView alarmHistoryListRecyclerView = findViewById(R.id.alarmHistoryListRecyclerView);
        alarmHistoryListRecyclerView.setLayoutManager(new LinearLayoutManager(this));
        alarmHistoryListRecyclerView.setHasFixedSize(true);

        final AlarmHistoryListAdapter alarmHistoryListAdapter  = new AlarmHistoryListAdapter();
        alarmHistoryListRecyclerView.setAdapter(alarmHistoryListAdapter);

        backToMainActivityButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        mAccelerometerViewModel = new ViewModelProvider(this).get(AccelerometerViewModel.class);
        mAccelerometerViewModel.init();
        mAccelerometerViewModel.getConnectedDeviceList().observe(this, new Observer<List<Device>>() {
            @Override
            public void onChanged(List<Device> devices) {
                deviceListAdapter.setDevices(devices);
                Toast.makeText(AccelerometerActivity.this, "onChanged", Toast.LENGTH_LONG).show();
            }
        });

        mAccelerometerViewModel.getAlarmHistoryList().observe(this, new Observer<List<Device>>() {
            @Override
            public void onChanged(List<Device> devices) {
                alarmHistoryListAdapter.setAlarmHistoryList(devices);
            }
        });

        mAccelerometerViewModel.getMovementDetectedValue().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean aBoolean) {
                if (aBoolean) {
                    mAccelerometerViewModel.updateDevice(mAccelerometerViewModel.getLocalDeviceUUID(), mAccelerometerViewModel.getSystemTimeStamp(), true, "Accelerometer", 10);
                }
                else {
                    mAccelerometerViewModel.updateDevice(mAccelerometerViewModel.getLocalDeviceUUID(), mAccelerometerViewModel.getSystemTimeStamp(), false, "Accelerometer", 0);
                }

            }
        });

        accelerometerManager = (SensorManager) getSystemService(SENSOR_SERVICE);
        if (accelerometerManager.getSensorList(Sensor.TYPE_GYROSCOPE).size() == 0) {
            accelerometerSensor = null;
            Toast.makeText(AccelerometerActivity.this, "No accelerometer sensor available", Toast.LENGTH_LONG).show();
        }
        else {
            accelerometerSensor = accelerometerManager.getSensorList(Sensor.TYPE_GYROSCOPE).get(0);
        }
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        StringBuilder sb = new StringBuilder();
        sb.append("x=")
                .append(event.values[0])
                .append("\ny=")
                .append(event.values[1])
                .append("\nz=")
                .append(event.values[2]);
        accelerometerDataTV.setText(sb.toString());
        mAccelerometerViewModel.addValueToGesamtBE(event.values[0] + event.values[1] + event.values[2]);
        mAccelerometerViewModel.meanValueCalculation();
        mAccelerometerViewModel.calibrateAccelerometerSensor();
        mAccelerometerViewModel.checkAlarmCondition();

    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (accelerometerSensor != null) {
            if (accelerometerManager.registerListener(this, accelerometerSensor, SensorManager.SENSOR_DELAY_GAME)) {
                Toast.makeText(AccelerometerActivity.this, "We registered to the sensor", Toast.LENGTH_LONG).show();
            } else {
                Toast.makeText(AccelerometerActivity.this, "Registration did not work", Toast.LENGTH_LONG).show();
            }
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (accelerometerSensor != null) {
            accelerometerManager.unregisterListener(this, accelerometerSensor);
        }
    }
}