package com.example.greenwatch.viewmodels;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import com.example.greenwatch.models.Device;
import com.example.greenwatch.repositories.DeviceRepository;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class AccelerometerViewModel extends ViewModel implements ViewModelInterface {

    private MutableLiveData<List<Device>> mDeviceList;
    private MutableLiveData<Boolean> mMovementDetected = new MutableLiveData<>();
    private MutableLiveData<List<Device>> mAlarmHistoryList;
    private DeviceRepository mDeviceRepository;
    private ArrayList<Float> Gesamt_be;
    int arraySize = 500;
    int functionCallCount;
    float meanValue;
    float offsetValue;
    float thresholdValue;
    boolean startMeasuring;

    @Override
    public void init() {
        if (mDeviceList == null) {
            mDeviceRepository = DeviceRepository.getInstance();
            mDeviceList = mDeviceRepository.getConnectedDeviceList();
            mAlarmHistoryList = mDeviceRepository.getAlarmHistoryDeviceList();
        }
        initGesamtBE();
        mMovementDetected.setValue(false);
        functionCallCount = 0;
        meanValue = 0f;
        offsetValue = 0.1f;
        thresholdValue = 0.15f;
        startMeasuring = false;
    }

    public void initGesamtBE() {
        Gesamt_be = new ArrayList<>(arraySize);
        for (int i = 0; i < arraySize; i++) {
            Gesamt_be.add(0f);
        }
    }

    public void addValueToGesamtBE(float newValue) {
        if (Gesamt_be.size() == arraySize) {
            Gesamt_be.remove(Gesamt_be.size() -1);
        }
        Gesamt_be.add(0, newValue);
        functionCallCount++;
    }

    public void meanValueCalculation() {
        for (float element : Gesamt_be) {
            meanValue += Math.abs(element);
        }
        meanValue = meanValue/arraySize;
    }

    public void calibrateAccelerometerSensor() {
        if (functionCallCount <= arraySize) {
            offsetValue = meanValue;
        }
        else {
            startMeasuring = true;
        }
    }

    public void checkAlarmCondition() {
        if (meanValue > (thresholdValue + offsetValue) && startMeasuring && !mMovementDetected.getValue()) {
            mMovementDetected.setValue(true);
        }
        else if (meanValue < (thresholdValue + offsetValue) && mMovementDetected.getValue()){
            mMovementDetected.setValue(false);
        }
    }

    public LiveData<Boolean> getMovementDetectedValue() {
        return mMovementDetected;
    }

    @Override
    public LiveData<List<Device>> getConnectedDeviceList() {
        return mDeviceList;
    }

    @Override
    public LiveData<List<Device>> getAlarmHistoryList() {
        return mAlarmHistoryList;
    }

    @Override
    public void updateDevice(String deviceID, String timeStamp, boolean sensorStatus, String sensorType, int sensorMassage) {
        mDeviceRepository.updateDevice(deviceID, timeStamp, sensorStatus, sensorType, sensorMassage);
    }

    @Override
    public void setTimeStamp(String deviceID, String timeStamp) {
        mDeviceRepository.setTimeStamp(deviceID, timeStamp);
    }

    @Override
    public String getTimeStamp(String deviceID) {
        return mDeviceRepository.getTimeStamp(deviceID);
    }

    @Override
    public void setDeviceID(String deviceID, String newDeviceID) {
        mDeviceRepository.setDeviceID(deviceID, newDeviceID);
    }

    @Override
    public String getDeviceID(String deviceID) {
        return mDeviceRepository.getDeviceID(deviceID);
    }

    @Override
    public void setSensorStatus(String deviceID, boolean sensorStatus) {
        mDeviceRepository.setSensorStatus(deviceID, sensorStatus);
    }

    @Override
    public boolean getSensorStatus(String deviceID) {
        return mDeviceRepository.getSensorStatus(deviceID);
    }

    @Override
    public void setSensorType(String deviceID, String sensorType) {
        mDeviceRepository.setSensorType(deviceID, sensorType);
    }

    @Override
    public String getSensorType(String deviceID) {
        return mDeviceRepository.getSensorType(deviceID);
    }

    @Override
    public void setSensorMassage(String deviceID, int sensorMessage) {
        mDeviceRepository.setSensorMassage(deviceID, sensorMessage);
    }

    @Override
    public int getSensorMassage(String deviceID) {
        return mDeviceRepository.getSensorMassage(deviceID);
    }

    @Override
    public String getLocalDeviceUUID() {
        return mDeviceRepository.getLocalDeviceUUID();
    }

    @Override
    public String getSystemTimeStamp() {
        return mDeviceRepository.getSystemTimeStamp();
    }
}
