package com.example.greenwatch.viewmodels;

import android.content.Context;

import androidx.camera.lifecycle.ProcessCameraProvider;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import com.example.greenwatch.models.Device;
import com.example.greenwatch.repositories.DeviceRepository;
import com.example.greenwatch.sensors.CameraSensor;

import java.util.List;

public class VideodetectionViewModel extends ViewModel implements ViewModelInterface {

    private MutableLiveData<List<Device>> mDeviceList;
    private MutableLiveData<Boolean> mVideoAlarmDetected;
    private MutableLiveData<List<Device>> mAlarmHistoryList;
    private DeviceRepository mDeviceRepository;
    private CameraSensor mCameraSensor;


    public LiveData<Boolean> getVideoMovementDetectedValue() {
        return mVideoAlarmDetected;
    }

    @Override
    public void init() {
        if (mDeviceRepository == null) {
            mDeviceRepository = DeviceRepository.getInstance();
        }
        if (mCameraSensor == null) {
            mCameraSensor = CameraSensor.getInstance();
        }
        if (mDeviceList == null) {
            mDeviceList = mDeviceRepository.getConnectedDeviceList();
        }
        if (mAlarmHistoryList == null) {
            mAlarmHistoryList = mDeviceRepository.getAlarmHistoryDeviceList();
        }
        if (mVideoAlarmDetected == null) {
            mVideoAlarmDetected = mCameraSensor.getVideoAlarmDetectedValue();
        }
    }

    public void bindImageAnalysis(ProcessCameraProvider cameraProvider, LifecycleOwner lifecycleOwner, Context context) {
        mCameraSensor.bindImageAnalysis(cameraProvider, lifecycleOwner, context);
    }

    public LiveData<Boolean> getVideoAlarmDetectedValue() {
        return mVideoAlarmDetected;
    }

    @Override
    public LiveData<List<Device>> getConnectedDeviceList() {
        return mDeviceList;
    }

    @Override
    public LiveData<List<Device>> getAlarmHistoryList() {
        return mAlarmHistoryList;
    }

    @Override
    public void updateDevice(String deviceID, String timeStamp, boolean sensorStatus, String sensorType, int sensorMassage) {
        mDeviceRepository.updateDevice(deviceID, timeStamp, sensorStatus, sensorType, sensorMassage);
    }

    @Override
    public void setTimeStamp(String deviceID, String timeStamp) {
        mDeviceRepository.setTimeStamp(deviceID, timeStamp);
    }

    @Override
    public String getTimeStamp(String deviceID) {
        return mDeviceRepository.getTimeStamp(deviceID);
    }

    @Override
    public void setDeviceID(String deviceID, String newDeviceID) {
        mDeviceRepository.setDeviceID(deviceID, newDeviceID);
    }

    @Override
    public String getDeviceID(String deviceID) {
        return mDeviceRepository.getDeviceID(deviceID);
    }

    @Override
    public void setSensorStatus(String deviceID, boolean sensorStatus) {
        mDeviceRepository.setSensorStatus(deviceID, sensorStatus);
    }

    @Override
    public boolean getSensorStatus(String deviceID) {
        return mDeviceRepository.getSensorStatus(deviceID);
    }

    @Override
    public void setSensorType(String deviceID, String sensorType) {
        mDeviceRepository.setSensorType(deviceID, sensorType);
    }

    @Override
    public String getSensorType(String deviceID) {
        return mDeviceRepository.getSensorType(deviceID);
    }

    @Override
    public void setSensorMassage(String deviceID, int sensorMessage) {
        mDeviceRepository.setSensorMassage(deviceID, sensorMessage);
    }

    @Override
    public int getSensorMassage(String deviceID) {
        return mDeviceRepository.getSensorMassage(deviceID);
    }

    @Override
    public String getLocalDeviceUUID() {
        return mDeviceRepository.getLocalDeviceUUID();
    }

    @Override
    public String getSystemTimeStamp() {
        return mDeviceRepository.getSystemTimeStamp();
    }
}
